@@ -4,6 +4,7 @@
    Copyright (C) 2007-2009 Lukas Appelhans <l.appelhans@gmx.de>
    Copyright (C) 2008 Urs Wolfer <uwolfer @ kde.org>
    Copyright (C) 2008 Dario Freddi <drf54321@gmail.com>
+   Copyright (C) 2009 Matthias Fuchs <mat69@gmx.net>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -14,6 +15,7 @@
 #include "core/kget.h"
 
 #include "mainwindow.h"
+#include "core/mostlocalurl.h"
 #include "core/transfer.h"
 #include "core/transferdatasource.h"
 #include "core/transfergroup.h"
@@ -27,7 +29,6 @@
 #include "settings.h"
 #include "core/transferhistorystore.h"
 #include <iostream>
-#include <kio/netaccess.h>
 #include <kinputdialog.h>
 #include <kfiledialog.h>
 #include <kmessagebox.h>
@@ -41,6 +42,7 @@
 #include <KPluginInfo>
 #include <KComboBox>
 #include <KConfigDialog>
+#include <KSaveFile>
 #include <KWindowSystem>
 
 #include <QTextStream>
@@ -60,7 +62,6 @@
     #include <QDBusInterface>
     #include <QDBusPendingCall>
     #include <kworkspace/kworkspace.h>
-    #include <solid/control/powermanager.h>
     #include <solid/powermanagement.h>
 #endif
 
@@ -178,7 +179,7 @@ QStringList KGet::transferGroupNames()
 TransferHandler * KGet::addTransfer(KUrl srcUrl, QString destDir, QString suggestedFileName, // krazy:exclude=passbyvalue
                                     QString groupName, bool start)
 {
-    srcUrl = KIO::NetAccess::mostLocalUrl(srcUrl, m_mainWindow);
+    srcUrl = mostLocalUrl(srcUrl);
     // Note: destDir may actually be a full path to a file :-(
     kDebug(5001) << "Source:" << srcUrl.url() << ", dest: " << destDir << ", sugg file: " << suggestedFileName << endl;
 
@@ -288,7 +289,7 @@ const QList<TransferHandler *> KGet::addTransfer(KUrl::List srcUrls, QString des
 
     for(; it!=itEnd ; ++it)
     {
-        *it = KIO::NetAccess::mostLocalUrl(*it, m_mainWindow);
+        *it = mostLocalUrl(*it);
         if ( isValidSource( *it ) )
             urlsToDownload.append( *it );
     }
@@ -321,6 +322,7 @@ const QList<TransferHandler *> KGet::addTransfer(KUrl::List srcUrls, QString des
     {
         if (destDir.isEmpty())
         {
+            //TODO only use groupsFromExceptions if that is allowed in the settings
             QList<TransferGroupHandler*> list = groupsFromExceptions(*it);
             if (!list.isEmpty()) {
                 destDir = list.first()->defaultFolder();
@@ -336,21 +338,47 @@ const QList<TransferHandler *> KGet::addTransfer(KUrl::List srcUrls, QString des
     }
 
     QList<TransferHandler*> transfers = createTransfers(data);
-    foreach (TransferHandler *transfer, transfers) {
-        KGet::showNotification(m_mainWindow, "added",
-                                i18n("<p>The following transfer has been added to the download list:</p><p style=\"font-size: small;\">\%1</p>", transfer->source().pathOrUrl()),
-                                "kget", i18n("Download added"));
+    if (!transfers.isEmpty()) {
+        QString urls = transfers[0]->source().pathOrUrl();
+        for (int i = 1; i < transfers.count(); ++i) {
+            urls += '\n' + transfers[i]->source().pathOrUrl();
+        }
+
+        QString message;
+        if (transfers.count() == 1) {
+            message = i18n("<p>The following transfer has been added to the download list:</p>");
+        } else {
+            message = i18n("<p>The following transfers have been added to the download list:</p>");
+        }
+        const QString content = QString("<p style=\"font-size: small;\">\%1</p>").arg(urls);
+        KGet::showNotification(m_mainWindow, "added", message + content, "kget", i18n("Download added"));
     }
+
     return transfers;
 }
 
 
-bool KGet::delTransfer(TransferHandler * transfer)
+bool KGet::delTransfer(TransferHandler * transfer, DeleteMode mode)
 {
-    return delTransfers(QList<TransferHandler*>() << transfer);
+    return delTransfers(QList<TransferHandler*>() << transfer, false, mode);
 }
 
-bool KGet::delTransfers(QList<TransferHandler*> handlers)
+bool KGet::delTransferSynchronously(TransferHandler* transfer, DeleteMode mode)
+{
+    return delTransfers(QList<TransferHandler*>() << transfer, true, mode);
+}
+
+bool KGet::delTransfers(const QList<TransferHandler*> &transfers, DeleteMode mode)
+{
+    return delTransfers(transfers, false, mode);
+}
+
+bool KGet::delTransfersSynchronously(const QList<TransferHandler*> &transfers, DeleteMode mode)
+{
+    return delTransfers(transfers, true, mode);
+}
+
+bool KGet::delTransfers(const QList<TransferHandler*> &handlers, bool synchronously, DeleteMode mode)
 {
     if (!m_store) {
         m_store = TransferHistoryStore::getStore();
@@ -365,7 +393,14 @@ bool KGet::delTransfers(QList<TransferHandler*> handlers)
         // TransferHandler deinitializations
         handler->destroy();
         // Transfer deinitializations (the deinit function is called by the destroy() function)
-        transfer->destroy();
+        if (mode == AutoDelete) {
+            Transfer::DeleteOptions o = Transfer::DeleteTemporaryFiles;
+            if (transfer->status() != Job::Finished && transfer->status() != Job::FinishedKeepAlive)
+                o |= Transfer::DeleteFiles;
+            transfer->destroy(o, synchronously);
+        } else {
+            transfer->destroy((Transfer::DeleteTemporaryFiles | Transfer::DeleteFiles), synchronously);
+        }
     }
     m_store->saveItems(historyItems);
 
@@ -374,6 +409,7 @@ bool KGet::delTransfers(QList<TransferHandler*> handlers)
     return true;
 }
 
+
 void KGet::moveTransfer(TransferHandler * transfer, const QString& groupName)
 {
   Q_UNUSED(transfer)
@@ -387,7 +423,7 @@ void KGet::redownloadTransfer(TransferHandler * transfer)
      QString dest = transfer->dest().url();
      QString destFile = transfer->dest().fileName();
 
-     KGet::delTransfer(transfer);
+     KGet::delTransferSynchronously(transfer);
      KGet::addTransfer(src, dest, destFile, group, true);
 }
 
@@ -444,8 +480,11 @@ QList<TransferHandler *> KGet::finishedTransfers()
     return finishedTransfers;
 }
 
-QList<TransferGroupHandler *> KGet::selectedTransferGroups()
+QList<TransferGroupHandler *> KGet::selectedTransferGroups(bool *mainSelected)
 {
+    if (mainSelected) {
+        *mainSelected = false;
+    }
     QList<TransferGroupHandler *> selectedTransferGroups;
 
     QModelIndexList selectedIndexes = m_selectionModel->selectedRows();
@@ -453,8 +492,13 @@ QList<TransferGroupHandler *> KGet::selectedTransferGroups()
     foreach(const QModelIndex &currentIndex, selectedIndexes)
     {
         ModelItem * item = m_transferTreeModel->itemFromIndex(currentIndex);
-        if (item->isGroup())
-            selectedTransferGroups.append(item->asGroup()->groupHandler());
+        if (item->isGroup()) {
+            TransferGroupHandler *group = item->asGroup()->groupHandler();
+            if (mainSelected && group->name() == i18n("My Downloads")) {
+                *mainSelected = true;
+            }
+            selectedTransferGroups.append(group);
+        }
     }
 
     return selectedTransferGroups;
@@ -554,7 +598,7 @@ void KGet::save( QString filename, bool plain ) // krazy:exclude=passbyvalue
     if(filename.isEmpty())
         filename = KStandardDirs::locateLocal("appdata", "transfers.kgt");
 
-    QFile file(filename);
+    KSaveFile file(filename);
     if ( !file.open( QIODevice::WriteOnly ) )
     {
         //kWarning(5001)<<"Unable to open output file when saving";
@@ -585,7 +629,7 @@ void KGet::save( QString filename, bool plain ) // krazy:exclude=passbyvalue
         QTextStream stream( &file );
         doc.save( stream, 2 );
     }
-    file.close();
+    file.finalize();
 }
 
 QList<TransferFactory*> KGet::factories()
@@ -616,7 +660,7 @@ void KGet::setSchedulerRunning(bool running)
 
 bool KGet::schedulerRunning()
 {
-    return (m_scheduler->countRunningJobs() > 0);
+    return (m_scheduler->hasRunningJobs());
 }
 
 void KGet::setSuspendScheduler(bool isSuspended)
@@ -701,36 +745,47 @@ QList<TransferGroupHandler*> KGet::groupsFromExceptions(const KUrl &filename)
 {
     QList<TransferGroupHandler*> handlers;
     foreach (TransferGroupHandler * handler, allTransferGroups()) {
-        QStringList patterns = handler->regExp().pattern().split(',');//FIXME 4.5 add a tooltip: "Enter a list of foo separated by ," and then do split(i18nc("used as separator in a list, translate to the same thing you translated \"Enter a list of foo separated by ,\"", ","))
-        foreach (const QString &pattern, patterns) {
-            QRegExp regExp = QRegExp(pattern.trimmed());
-
-            //try with Regular Expression first
-            regExp.setPatternSyntax(QRegExp::RegExp2);
-            regExp.setCaseSensitivity(Qt::CaseInsensitive);
-            if (regExp.exactMatch(filename.url())) {
-                handlers.append(handler);
-                break;
-            }
-
-            //now try with wildcards
-            if (!regExp.pattern().isEmpty() && !regExp.pattern().contains('*')) {
-                regExp.setPattern('*' + regExp.pattern());
-            }
-
-            regExp.setPatternSyntax(QRegExp::Wildcard);
-            regExp.setCaseSensitivity(Qt::CaseInsensitive);
-
-            if (regExp.exactMatch(filename.url())) {
-                handlers.append(handler);
-                break;
-            }
+        const QStringList patterns = handler->regExp().pattern().split(',');//FIXME 4.5 add a tooltip: "Enter a list of foo separated by ," and then do split(i18nc("used as separator in a list, translate to the same thing you translated \"Enter a list of foo separated by ,\"", ","))
+        if (matchesExceptions(filename, patterns)) {
+            handlers.append(handler);
         }
     }
 
     return handlers;
 }
 
+bool KGet::matchesExceptions(const KUrl &sourceUrl, const QStringList &patterns)
+{
+    foreach (const QString &pattern, patterns) {
+        const QString trimmedPattern = pattern.trimmed();
+        if (trimmedPattern.isEmpty()) {
+            continue;
+        }
+        QRegExp regExp = QRegExp(trimmedPattern);
+
+        //try with Regular Expression first
+        regExp.setPatternSyntax(QRegExp::RegExp2);
+        regExp.setCaseSensitivity(Qt::CaseInsensitive);
+        if (regExp.exactMatch(sourceUrl.url())) {
+            return true;
+        }
+
+        //now try with wildcards
+        if (!regExp.pattern().isEmpty() && !regExp.pattern().contains('*')) {
+            regExp.setPattern('*' + regExp.pattern());
+        }
+
+        regExp.setPatternSyntax(QRegExp::Wildcard);
+        regExp.setCaseSensitivity(Qt::CaseInsensitive);
+
+        if (regExp.exactMatch(sourceUrl.url())) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 void KGet::setGlobalDownloadLimit(int limit)
 {
     m_scheduler->setDownloadLimit(limit);
@@ -763,10 +818,11 @@ void KGet::calculateGlobalUploadLimit()
 TransferTreeModel * KGet::m_transferTreeModel;
 TransferTreeSelectionModel * KGet::m_selectionModel;
 QList<TransferFactory *> KGet::m_transferFactories;
-TransferGroupScheduler * KGet::m_scheduler = new TransferGroupScheduler();
+TransferGroupScheduler * KGet::m_scheduler = 0;
 MainWindow * KGet::m_mainWindow = 0;
 KUiServerJobs * KGet::m_jobManager = 0;
 TransferHistoryStore * KGet::m_store = 0;
+bool KGet::m_hasConnection = true;
 #ifdef HAVE_NEPOMUK
     NepomukController *KGet::m_nepomukController = 0;
 #endif
@@ -778,6 +834,7 @@ KGet::KGet()
     Nepomuk::ResourceManager::instance()->init();
 #endif
 
+    m_scheduler = new TransferGroupScheduler();
     m_transferTreeModel = new TransferTreeModel(m_scheduler);
     m_selectionModel = new TransferTreeSelectionModel(m_transferTreeModel);
 
@@ -785,8 +842,12 @@ KGet::KGet()
                      m_jobManager,        SLOT(slotTransfersAdded(QList<TransferHandler*>)));
     QObject::connect(m_transferTreeModel, SIGNAL(transfersAboutToBeRemovedEvent(QList<TransferHandler*>)),
                      m_jobManager,        SLOT(slotTransfersAboutToBeRemoved(QList<TransferHandler*>)));
-    QObject::connect(m_transferTreeModel, SIGNAL(transfersChangedEvent(QMap<TransferHandler *, Transfer::ChangesFlags>)),
-                     m_jobManager,        SLOT(slotTransfersChanged(QMap<TransferHandler *, Transfer::ChangesFlags>)));
+    QObject::connect(m_transferTreeModel, SIGNAL(transfersChangedEvent(QMap<TransferHandler*,Transfer::ChangesFlags>)),
+                     m_jobManager,        SLOT(slotTransfersChanged(QMap<TransferHandler*,Transfer::ChangesFlags>)));
+
+    //check if there is a connection
+    const Solid::Networking::Status status = Solid::Networking::status();
+    KGet::setHasNetworkConnection((status == Solid::Networking::Connected) || (status == Solid::Networking::Unknown));
             
     //Load all the available plugins
     loadPlugins();
@@ -825,6 +886,7 @@ QList<TransferHandler*> KGet::createTransfers(const QList<TransferData> &dataIte
     QList<bool> start;
     QHash<TransferGroup*, QList<Transfer*> > groups;
 
+    QStringList urlsFailed;
     foreach (const TransferData &data, dataItems) {
         kDebug(5001) << "srcUrl=" << data.src << " destUrl=" << data.dest << " group=" << data.groupName;
 
@@ -848,13 +910,26 @@ QList<TransferHandler*> KGet::createTransfers(const QList<TransferData> &dataIte
             }
         }
         if (!newTransfer) {
-            KGet::showNotification(m_mainWindow, "error",
-                                   i18n("Unable to continue: KGet cannot download using this protocol."), "dialog-error", i18n("Protocol unsupported"));
-
+            urlsFailed << data.src.url();
             kWarning(5001) << "Warning! No plugin found to handle" << data.src;
         }
     }
 
+    //show urls that failed
+    if (!urlsFailed.isEmpty()) {
+        QString message = i18np("<p>The following URL cannot be downloaded, its protocol is not supported by KGet:</p>",              
+                                "<p>The following URLs cannot be downloaded, their protocols are not supported by KGet:</p>",
+                                urlsFailed.count());
+
+        QString content = urlsFailed.takeFirst();
+        foreach (const QString &url, urlsFailed) {
+            content += '\n' + url;
+        }
+        content = QString("<p style=\"font-size: small;\">\%1</p>").arg(content);
+
+        KGet::showNotification(m_mainWindow, "error", message + content, "dialog-error", i18n("Protocol unsupported"));
+    }
+
     //add the created transfers to the model and start them if specified
     QHash<TransferGroup*, QList<Transfer*> >::const_iterator it;
     QHash<TransferGroup*, QList<Transfer*> >::const_iterator itEnd = groups.constEnd();
@@ -979,7 +1054,7 @@ bool KGet::isValidSource(const KUrl &source)
                     KStandardGuiItem::no(), KStandardGuiItem::cancel())
                     == KMessageBox::Yes) {
                 transfer->stop();
-                KGet::delTransfer(transfer->handler());
+                KGet::delTransferSynchronously(transfer->handler());
                 return true;
             }
             else
@@ -992,7 +1067,7 @@ bool KGet::isValidSource(const KUrl &source)
                     KStandardGuiItem::no(), KStandardGuiItem::cancel())
                     == KMessageBox::Yes) {
                 transfer->stop();
-                KGet::delTransfer(transfer->handler());
+                KGet::delTransferSynchronously(transfer->handler());
                 return true;
             }
             else
@@ -1051,7 +1126,7 @@ KUrl KGet::getValidDestUrl(const KUrl& destDir, const KUrl &srcUrl)
                     KStandardGuiItem::no(), KStandardGuiItem::cancel())
                     == KMessageBox::Yes) {
                 existingTransferDest->stop();
-                KGet::delTransfer(existingTransferDest->handler());
+                KGet::delTransferSynchronously(existingTransferDest->handler());
                 //start = true;
             } else 
                 return KUrl();
@@ -1145,9 +1220,6 @@ void KGet::loadPlugins()
         }
     }
 
-    QList<KGetPlugin *>::ConstIterator it2 = pluginList.constBegin();
-    QList<KGetPlugin *>::ConstIterator it2End = pluginList.constEnd();
-
     foreach (KGetPlugin *plugin, pluginList)
     {
         m_transferFactories.append(qobject_cast<TransferFactory *>(plugin));
@@ -1156,6 +1228,28 @@ void KGet::loadPlugins()
     kDebug(5001) << "Number of factories = " << m_transferFactories.size();
 }
 
+
+void KGet::setHasNetworkConnection(bool hasConnection)
+{
+    kDebug(5001) << "Existing internet connection:" << hasConnection << "old:" << m_hasConnection;
+    if (hasConnection == m_hasConnection) {
+        return;
+    }
+    m_hasConnection = hasConnection;
+
+    if (hasConnection) {
+        KGet::showNotification(m_mainWindow, "notification",
+                               i18n("Internet connection established, resuming transfers."),
+                               "dialog-info");
+
+    } else {
+        KGet::showNotification(m_mainWindow, "notification",
+                               i18n("No internet connection, stopping transfers."),
+                               "dialog-info");
+    }
+    m_scheduler->setHasNetworkConnection(hasConnection);
+}
+
 KGetPlugin * KGet::createPluginFromService( const KService::Ptr &service )
 {
     //try to load the specified library
@@ -1214,12 +1308,15 @@ GenericObserver::GenericObserver(QObject *parent)
     connect(KGet::model(), SIGNAL(groupAddedEvent(TransferGroupHandler*)), SLOT(groupAddedEvent(TransferGroupHandler*)));
     connect(KGet::model(), SIGNAL(transfersRemovedEvent(QList<TransferHandler*>)),
                            SLOT(transfersRemovedEvent(QList<TransferHandler*>)));
-    connect(KGet::model(), SIGNAL(transfersChangedEvent(QMap<TransferHandler*, Transfer::ChangesFlags>)), 
-                           SLOT(transfersChangedEvent(QMap<TransferHandler*, Transfer::ChangesFlags>)));
-    connect(KGet::model(), SIGNAL(groupsChangedEvent(QMap<TransferGroupHandler*, TransferGroup::ChangesFlags>)), 
-                           SLOT(groupsChangedEvent(QMap<TransferGroupHandler*, TransferGroup::ChangesFlags>)));
-    connect(KGet::model(), SIGNAL(transferMovedEvent(TransferHandler *, TransferGroupHandler *)),
-                           SLOT(transferMovedEvent(TransferHandler *, TransferGroupHandler *)));
+    connect(KGet::model(), SIGNAL(transfersChangedEvent(QMap<TransferHandler*,Transfer::ChangesFlags>)), 
+                           SLOT(transfersChangedEvent(QMap<TransferHandler*,Transfer::ChangesFlags>)));
+    connect(KGet::model(), SIGNAL(groupsChangedEvent(QMap<TransferGroupHandler*,TransferGroup::ChangesFlags>)), 
+                           SLOT(groupsChangedEvent(QMap<TransferGroupHandler*,TransferGroup::ChangesFlags>)));
+    connect(KGet::model(), SIGNAL(transferMovedEvent(TransferHandler*,TransferGroupHandler*)),
+                           SLOT(transferMovedEvent(TransferHandler*,TransferGroupHandler*)));
+    connect(Solid::Networking::notifier(), SIGNAL(statusChanged(Solid::Networking::Status)),
+                         this, SLOT(slotNetworkStatusChanged(Solid::Networking::Status)));
+
 }
 
 GenericObserver::~GenericObserver()
@@ -1266,11 +1363,13 @@ void GenericObserver::requestSave()
 {
     if (!m_save) {
         m_save = new QTimer(this);
-        m_save->setSingleShot(true);
         m_save->setInterval(5000);
         connect(m_save, SIGNAL(timeout()), this, SLOT(slotSave()));
     }
 
+    //save regularly if there are running jobs
+    m_save->setSingleShot(!KGet::m_scheduler->hasRunningJobs());
+
     if (!m_save->isActive()) {
         m_save->start();
     }
@@ -1301,11 +1400,23 @@ void GenericObserver::transfersChangedEvent(QMap<TransferHandler*, Transfer::Cha
                 KGet::showNotification(KGet::m_mainWindow, "started",
                                        i18n("<p>The following transfer has been started:</p><p style=\"font-size: small;\">\%1</p>", transfer->source().pathOrUrl()),
                                        "kget", i18n("Download started"));
+            } else if (transfer->status() == Job::Aborted && transfer->error().type != Job::AutomaticRetry) {
+                KNotification * notification = KNotification::event("error", i18n("Error"), i18n("<p>There has been an error in the following transfer:</p><p style=\"font-size: small;\">\%1</p>"
+                                            "<p>The error message is:</p><p style=\"font-size: small;\">\%2</p>", transfer->source().pathOrUrl(), transfer->error().text), 
+                                             transfer->error().pixmap, KGet::m_mainWindow, KNotification::CloseOnTimeout);
+                if (transfer->error().type == Job::ManualSolve) {
+                    m_notifications.insert(notification, transfer);
+                    notification->setActions(QStringList() << i18n("Resolve"));
+                    connect(notification, SIGNAL(action1Activated()), SLOT(slotResolveTransferError()));
+                    connect(notification, SIGNAL(closed()), SLOT(slotNotificationClosed()));
+                }
             }
         }
 
-        if (transferFlags & Transfer::Tc_Status)
+        if (transferFlags & Transfer::Tc_Status) {
             checkSysTray = true;
+            requestSave();
+        }
 
         if (transferFlags & Transfer::Tc_Percent) {
             transfer->group()->setGroupChange(TransferGroup::Gc_Percent, true);
@@ -1331,7 +1442,9 @@ void GenericObserver::transfersChangedEvent(QMap<TransferHandler*, Transfer::Cha
     if (checkSysTray)
         KGet::checkSystemTray();
 
-    if (Settings::afterFinishActionEnabled() && allFinished)
+    //only perform after finished actions if actually the status changed (that is the
+    //case if checkSysTray is set to true)
+    if (checkSysTray && Settings::afterFinishActionEnabled() && allFinished)
     {
         kDebug(5001) << "All finished";
         KNotification *notification = 0;
@@ -1378,10 +1491,34 @@ void GenericObserver::transfersChangedEvent(QMap<TransferHandler*, Transfer::Cha
     }
 }
 
+void GenericObserver::slotResolveTransferError()
+{
+    KNotification * notification = static_cast<KNotification*>(QObject::sender());
+    if (notification) {
+        TransferHandler * handler = m_notifications[notification];
+        kDebug() << "Resolve error for" << handler->source().pathOrUrl() << "with id" << handler->error().id;
+        handler->resolveError(handler->error().id);
+        m_notifications.remove(notification);
+    }
+}
+
+void GenericObserver::slotNotificationClosed()
+{
+    kDebug() << "Remove notification";
+    KNotification * notification = static_cast<KNotification*>(QObject::sender());
+    if (notification)
+        m_notifications.remove(notification);
+}
+
+void GenericObserver::slotNetworkStatusChanged(const Solid::Networking::Status &status)
+{
+    KGet::setHasNetworkConnection((status == Solid::Networking::Connected) || (status == Solid::Networking::Unknown));
+}
+
 void GenericObserver::groupsChangedEvent(QMap<TransferGroupHandler*, TransferGroup::ChangesFlags> groups)
 {
     bool recalculate = false;
-    foreach (TransferGroup::ChangesFlags flags, groups)
+    foreach (const TransferGroup::ChangesFlags &flags, groups)
     {
         if (flags & TransferGroup::Gc_Percent || flags & TransferGroup::Gc_Status) {
             recalculate = true;
@@ -1406,9 +1543,8 @@ bool GenericObserver::allTransfersFinished()
             if ((transfer->status() != Job::Finished) && (transfer->status() != Job::FinishedKeepAlive)) {
                 quitFlag = false;
             }
-            if (((transfer->status() == Job::Finished) && (transfer->startStatus() != Job::Finished)) ||
-                ((transfer->status() == Job::FinishedKeepAlive) && (transfer->startStatus() != Job::FinishedKeepAlive)))
-            {
+            if ((transfer->status() == Job::Finished || transfer->status() == Job::FinishedKeepAlive) &&
+                (transfer->startStatus() != Job::Finished && transfer->startStatus() != Job::FinishedKeepAlive)) {
                 allWereFinished = false;
             }
         }
@@ -1444,15 +1580,21 @@ void GenericObserver::slotAfterFinishAction()
                         KWorkSpace::ShutdownModeForceNow);
             break;
         case KGet::Hibernate: {
-            QDBusConnection dbus(QDBusConnection::sessionBus());
-            QDBusInterface iface("org.kde.kded", "/modules/powerdevil", "org.kde.PowerDevil", dbus);
-            iface.asyncCall("suspend", Solid::Control::PowerManager::ToDisk);
+           QDBusMessage call;
+           call = QDBusMessage::createMethodCall("org.kde.Solid.PowerManagement",
+                                                 "/org/kde/Solid/PowerManagement",
+                                                 "org.kde.Solid.PowerManagement",
+                                                 "suspendToRam");
+           QDBusConnection::sessionBus().asyncCall(call);
             break;
         }
         case KGet::Suspend: {
-            QDBusConnection dbus(QDBusConnection::sessionBus());
-            QDBusInterface iface("org.kde.kded", "/modules/powerdevil", "org.kde.PowerDevil", dbus);
-            iface.asyncCall("suspend", Solid::Control::PowerManager::ToRam);
+           QDBusMessage call;
+           call = QDBusMessage::createMethodCall("org.kde.Solid.PowerManagement",
+                                                 "/org/kde/Solid/PowerManagement",
+                                                 "org.kde.Solid.PowerManagement",
+                                                 "suspendToDisk");
+           QDBusConnection::sessionBus().asyncCall(call);
             break;
         }
     #endif
