@@ -11,30 +11,34 @@
 */
 
 #include "metalink.h"
+#include "fileselectiondlg.h"
 #include "metalinksettings.h"
-#include "ui_fileselection.h"
 
 #include "core/kget.h"
 #include "core/transfergroup.h"
 #include "core/download.h"
 #include "core/transferdatasource.h"
 #include "core/filemodel.h"
+#include "core/urlchecker.h"
 #include "core/verifier.h"
+#include "core/signature.h"
 #ifdef HAVE_NEPOMUK
-#include "metanepomukhandler.h"
+#include "core/nepomukhandler.h"
+#include <Nepomuk/Variant>
 #endif //HAVE_NEPOMUK
 
 #include <KIconLoader>
 #include <KIO/DeleteJob>
 #include <KIO/NetAccess>
+#include <KIO/RenameDialog>
 #include <KLocale>
 #include <KMessageBox>
 #include <KDebug>
 #include <KDialog>
 #include <KStandardDirs>
 
-#include <QDomElement>
-#include <QSortFilterProxyModel>
+#include <QtCore/QFile>
+#include <QtXml/QDomElement>
 
 Metalink::Metalink(TransferGroup * parent, TransferFactory * factory,
                          Scheduler * scheduler, const KUrl & source, const KUrl & dest,
@@ -42,13 +46,11 @@ Metalink::Metalink(TransferGroup * parent, TransferFactory * factory,
     : Transfer(parent, factory, scheduler, source, dest, e),
       m_fileModel(0),
       m_currentFiles(0),
+      m_metalinkJustDownloaded(false),
       m_ready(false),
       m_speedCount(0),
       m_tempAverageSpeed(0),
       m_averageSpeed(0)
-#ifdef HAVE_NEPOMUK
-      , m_nepHandler(0)
-#endif
 {
 }
 
@@ -56,38 +58,16 @@ Metalink::~Metalink()
 {
 }
 
-void Metalink::init()
-{
-#ifdef HAVE_NEPOMUK
-    if (!m_nepHandler)
-    {
-        m_nepHandler = new MetaNepomukHandler(this);
-        setNepomukHandler(m_nepHandler);
-    }
-#endif //HAVE_NEPOMUK
-
-    Transfer::init();
-}
-
 void Metalink::start()
 {
     kDebug(5001) << "metalink::start";
 
     if (!m_ready)
     {
-        if (m_localMetalinkLocation.isValid())
-        {
-            metalinkInit();
+        if (m_localMetalinkLocation.isValid() && metalinkInit()) {
             startMetalink();
-        }
-        else
-        {
-            Download *download = new Download(m_source, KStandardDirs::locateLocal("appdata", "metalinks/") + m_source.fileName());
-
-            setStatus(Job::Running, i18n("Downloading Metalink File...."), SmallIcon("document-save"));
-            setTransferChange(Tc_Status, true);
-
-            connect(download, SIGNAL(finishedSuccessfully(KUrl, QByteArray)), SLOT(metalinkInit(KUrl, QByteArray)));
+        } else {
+            downloadMetalink();
         }
     }
     else
@@ -96,33 +76,51 @@ void Metalink::start()
     }
 }
 
-void Metalink::metalinkInit(const KUrl &src, const QByteArray &data)
+void Metalink::downloadMetalink()
+{
+    m_metalinkJustDownloaded = true;
+
+    setStatus(Job::Stopped, i18n("Downloading Metalink File...."), SmallIcon("document-save"));
+    setTransferChange(Tc_Status, true);
+    Download *download = new Download(m_source, QString(KStandardDirs::locateLocal("appdata", "metalinks/") + m_source.fileName()));
+    connect(download, SIGNAL(finishedSuccessfully(KUrl,QByteArray)), SLOT(metalinkInit(KUrl,QByteArray)));
+}
+
+bool Metalink::metalinkInit(const KUrl &src, const QByteArray &data)
 {
     kDebug(5001);
-    bool justDownloaded = !m_localMetalinkLocation.isValid();
-    if (!src.isEmpty())
-    {
+
+    if (!src.isEmpty()) {
         m_localMetalinkLocation = src;
     }
 
     //use the downloaded metalink-file data directly if possible
-    if (!data.isEmpty())
-    {
+    if (!data.isEmpty()) {
         KGetMetalink::HandleMetalink::load(data, &m_metalink);
     }
 
-    //TODO error message?
     //try to parse the locally stored metalink-file
-    if (!m_metalink.isValid())
-    {
+    if (!m_metalink.isValid() && m_localMetalinkLocation.isValid()) {
         KGetMetalink::HandleMetalink::load(m_localMetalinkLocation.toLocalFile(), &m_metalink);
     }
 
-    //error
-    if (!m_metalink.isValid())
-    {
-        kDebug(5001) << "Unknown error when trying to load the .metalink-file";
-        return;
+    if (!m_metalink.isValid()) {
+        kError(5001) << "Unknown error when trying to load the .metalink-file. Metalink is not valid.";
+        setStatus(Job::Aborted);
+        setTransferChange(Tc_Status, true);
+        return false;
+    }
+
+    //offers a dialog to download the newest version of a dynamic metalink
+     if ((m_source.isLocalFile() || !m_metalinkJustDownloaded) &&
+         m_metalink.dynamic && (UrlChecker::checkSource(m_metalink.origin) == UrlChecker::NoError)) {
+        if (KMessageBox::questionYesNo(0, i18n("A newer version of this Metalink might exist, do you want to download it?"),
+                                       i18n("Redownload Metalink")) == KMessageBox::Yes) {
+            m_localMetalinkLocation.clear();
+            m_source = m_metalink.origin;
+            downloadMetalink();
+            return false;
+        }
     }
 
     QList<KGetMetalink::File>::const_iterator it;
@@ -148,17 +146,16 @@ void Metalink::metalinkInit(const KUrl &src, const QByteArray &data)
         dataFactory->setMaxMirrorsUsed(MetalinkSettings::mirrorsPerFile());
 
 #ifdef HAVE_NEPOMUK
-        m_nepHandler->setFileMetaData(dest, *it);
+        nepomukHandler()->setProperties((*it).properties(), QList<KUrl>() << dest);
 #endif //HAVE_NEPOMUK
 
 //TODO compare available file size (<size>) with the sizes of the server while downloading?
 
-        connect(dataFactory, SIGNAL(totalSize(KIO::filesize_t)), this, SLOT(totalSizeChanged(KIO::filesize_t)));
-        connect(dataFactory, SIGNAL(processedSize(KIO::filesize_t)), this, SLOT(processedSizeChanged()));
-        connect(dataFactory, SIGNAL(speed(ulong)), this, SLOT(speedChanged()));
-        connect(dataFactory, SIGNAL(statusChanged(Job::Status)), this, SLOT(slotStatus(Job::Status)));
+        connect(dataFactory, SIGNAL(capabilitiesChanged()), this, SLOT(slotUpdateCapabilities()));
+        connect(dataFactory, SIGNAL(dataSourceFactoryChange(Transfer::ChangesFlags)), this, SLOT(slotDataSourceFactoryChange(Transfer::ChangesFlags)));
         connect(dataFactory->verifier(), SIGNAL(verified(bool)), this, SLOT(slotVerified(bool)));
         connect(dataFactory->signature(), SIGNAL(verified(int)), this, SLOT(slotSignatureVerified()));
+        connect(dataFactory, SIGNAL(log(QString,Transfer::LogLevel)), this, SLOT(setLog(QString,Transfer::LogLevel)));
 
         //add the DataSources
         for (int i = 0; i < urlList.size(); ++i)
@@ -176,7 +173,7 @@ void Metalink::metalinkInit(const KUrl &src, const QByteArray &data)
         }
         else
         {
-            dataFactory->verifier()->model()->addChecksums((*it).verification.hashes);
+            dataFactory->verifier()->addChecksums((*it).verification.hashes);
             foreach (const KGetMetalink::Pieces &pieces, (*it).verification.pieces) {
                 dataFactory->verifier()->addPartialChecksums(pieces.type, pieces.length, pieces.hashes);
             }
@@ -186,7 +183,7 @@ void Metalink::metalinkInit(const KUrl &src, const QByteArray &data)
             QHash<QString, QString>::const_iterator itEnd = signatures.constEnd();
             for (it = signatures.constBegin(); it != itEnd; ++it) {
                 if (it.key().toLower() == "pgp") {
-                    dataFactory->signature()->setSignature(*it);
+                    dataFactory->signature()->setAsciiDetatchedSignature(*it);
                 }
             }
 
@@ -199,64 +196,59 @@ void Metalink::metalinkInit(const KUrl &src, const QByteArray &data)
         m_dest = dest;
     }
 
-    if (!m_dataSourceFactory.size())
-    {
+    if (!m_dataSourceFactory.size()) {
+        //TODO make this via log in the future + do not display the KMessageBox
+        kWarning(5001) << "Download of" << m_source << "failed, no working URLs were found.";
         KMessageBox::error(0, i18n("Download failed, no working URLs were found."), i18n("Error"));
-        setStatus(Job::Aborted, i18n("An error occurred...."), SmallIcon("document-preview"));
+        setStatus(Job::Aborted);
+        setTransferChange(Tc_Status, true);
+        return false;
+    }
+
+    m_ready = !m_dataSourceFactory.isEmpty();
+    slotUpdateCapabilities();
+
+    //the metalink-file has just been downloaded, so ask the user to choose the files that
+    // should be downloaded
+    if (m_metalinkJustDownloaded) {
+        KDialog *dialog = new FileSelectionDlg(fileModel());
+        dialog->setAttribute(Qt::WA_DeleteOnClose);
+        connect(dialog, SIGNAL(finished(int)), this, SLOT(fileDlgFinished(int)));
+
+        dialog->show();
+    }
+
+    return true;
+}
+
+void Metalink::untickAllFiles()
+{
+    for (int row = 0; row < fileModel()->rowCount(); ++row) {
+        QModelIndex index = fileModel()->index(row, FileItem::File);
+        if (index.isValid()) {
+            fileModel()->setData(index, Qt::Unchecked, Qt::CheckStateRole);
+        }
+    }
+}
+
+void Metalink::fileDlgFinished(int result)
+{
+    //the dialog was not accepted untick every file, this ensures that the user does not
+    //press start by accident without first selecting the desired files
+    if (result != QDialog::Accepted) {
+        untickAllFiles();
+    }
+
+    filesSelected();
+
+    //no files selected to download or dialog rejected, stop the download
+    if (!m_numFilesSelected  || (result != QDialog::Accepted)) {
+        setStatus(Job::Stopped);
         setTransferChange(Tc_Status, true);
         return;
     }
 
-    m_ready = !m_dataSourceFactory.isEmpty();
-
-    //the metalink-file has just been downloaded, so ask the user to choose the files that
-    // should be downloaded
-    if (justDownloaded)
-    {
-        KDialog *dialog = new KDialog;
-        dialog->setAttribute(Qt::WA_DeleteOnClose);
-        QWidget *widget = new QWidget(dialog);
-        Ui::FileSelection ui;
-        ui.setupUi(widget);
-        QSortFilterProxyModel *proxy = new QSortFilterProxyModel(this);
-        proxy->setSourceModel(fileModel());
-        ui.treeView->setModel(proxy);
-        ui.treeView->sortByColumn(0, Qt::AscendingOrder);
-        ui.treeView->hideColumn(FileItem::Status);
-        ui.treeView->hideColumn(FileItem::ChecksumVerified);
-        ui.treeView->hideColumn(FileItem::SignatureVerified);
-        dialog->setMainWidget(widget);
-        dialog->setCaption(i18n("File Selection"));
-        dialog->setButtons(KDialog::Ok);
-        connect(dialog, SIGNAL(finished()), this, SLOT(filesSelected()));
-
-        dialog->show();
-    }
-}
-
-void Metalink::filesSelected()
-{
-    QModelIndexList files = fileModel()->fileIndexes(FileItem::File);
-    foreach (const QModelIndex &index, files)
-    {
-        const KUrl dest = fileModel()->getUrl(index);
-        const bool doDownload = index.data(Qt::CheckStateRole).toBool();
-        if (m_dataSourceFactory.contains(dest))
-        {
-            m_dataSourceFactory[dest]->setDoDownload(doDownload);
-        }
-    }
-
-    //make sure that the size, the downloaded size and the speed gets updated
-    totalSizeChanged(0);
-    processedSizeChanged();
-    speedChanged();
-
-    //some files may be set to download, so start them as long as the transfer is not stopped
-    if (status() != Job::Stopped)
-    {
-        startMetalink();
-    }
+    startMetalink();
 }
 
 void Metalink::startMetalink()
@@ -284,45 +276,48 @@ void Metalink::startMetalink()
                 break;
             }
         }
-
-#ifdef HAVE_NEPOMUK
-        m_nepHandler->setDestinations(files());
-#endif //HAVE_NEPOMUK
     }
 }
 
-void Metalink::deinit()
+void Metalink::deinit(Transfer::DeleteOptions options)
 {
-    if (status() != Job::Finished)//if the transfer is not finished, we delete the written files
-    {
-        foreach (DataSourceFactory *factory, m_dataSourceFactory)
-        {
+    foreach (DataSourceFactory *factory, m_dataSourceFactory) {
+        if (options & Transfer::DeleteFiles) {
             factory->deinit();
         }
     }//TODO: Ask the user if he/she wants to delete the *.part-file? To discuss (boom1992)
-    else
-    {
-        //in any case delete the files that should not be downloaded
-        foreach (DataSourceFactory *factory, m_dataSourceFactory)
-        {
-            if (!factory->doDownload())
-            {
-                factory->deinit();
-            }
-        }
-    }
 
-    if (m_localMetalinkLocation.isLocalFile())
+    if ((options & Transfer::DeleteTemporaryFiles) && m_localMetalinkLocation.isLocalFile())
     {
         KIO::Job *del = KIO::del(m_localMetalinkLocation, KIO::HideProgressInfo);
         KIO::NetAccess::synchronousRun(del, 0);
     }
 
 #ifdef HAVE_NEPOMUK
-    m_nepHandler->deinit();
+    nepomukHandler()->deinit();
 #endif //HAVE_NEPOMUK
 }
 
+void Metalink::synchronDeinit(Transfer::DeleteOptions options)
+{
+    foreach (DataSourceFactory *factory, m_dataSourceFactory) {
+        if (options & Transfer::DeleteFiles) {
+            factory->synchronDeinit();
+        }
+    }//TODO: Ask the user if he/she wants to delete the *.part-file? To discuss (boom1992)
+
+    if ((options & Transfer::DeleteTemporaryFiles) && m_localMetalinkLocation.isLocalFile())
+    {
+        KIO::Job *del = KIO::del(m_localMetalinkLocation, KIO::HideProgressInfo);
+        KIO::NetAccess::synchronousRun(del, 0);
+    }
+
+#ifdef HAVE_NEPOMUK
+    nepomukHandler()->deinit();
+#endif //HAVE_NEPOMUK
+}
+
+
 void Metalink::stop()
 {
     kDebug(5001) << "metalink::Stop";
@@ -336,12 +331,33 @@ void Metalink::stop()
     }
 }
 
-bool Metalink::isResumable() const
+void Metalink::slotDataSourceFactoryChange(Transfer::ChangesFlags change)
 {
-    return true;
+    if ((change & Tc_Status) | (change & Tc_TotalSize)) {
+        DataSourceFactory *factory = qobject_cast<DataSourceFactory*>(sender());
+        if (change & Tc_Status) {
+            bool changeStatus;
+            updateStatus(factory, &changeStatus);
+            if (!changeStatus) {
+                change &= ~Tc_Status;
+            }
+        }
+        if (change & Tc_TotalSize) {
+            recalculateTotalSize(factory);
+        }
+    }
+    if (change & Tc_DownloadedSize) {
+        recalculateProcessedSize();
+        change |= Tc_Percent;
+    }
+    if (change & Tc_DownloadSpeed) {
+        recalculateSpeed();
+    }
+
+    setTransferChange(change, true);
 }
 
-void Metalink::totalSizeChanged(KIO::filesize_t size)
+void Metalink::recalculateTotalSize(DataSourceFactory *sender)
 {
     m_totalSize = 0;
     foreach (DataSourceFactory *factory, m_dataSourceFactory)
@@ -352,21 +368,15 @@ void Metalink::totalSizeChanged(KIO::filesize_t size)
         }
     }
 
-    if (m_fileModel)
-    {
-        DataSourceFactory *factory = qobject_cast<DataSourceFactory*>(sender());
-        if (factory)
-        {
-            QModelIndex sizeIndex = m_fileModel->index(factory->dest(), FileItem::Size);
-            m_fileModel->setData(sizeIndex, static_cast<qlonglong>(size));
+    if (m_fileModel) {
+        if (sender) {
+            QModelIndex sizeIndex = m_fileModel->index(sender->dest(), FileItem::Size);
+            m_fileModel->setData(sizeIndex, static_cast<qlonglong>(sender->size()));
         }
     }
-
-    setTransferChange(Tc_TotalSize, true);
-    processedSizeChanged();
 }
 
-void Metalink::processedSizeChanged()
+void Metalink::recalculateProcessedSize()
 {
     m_downloadedSize = 0;
     foreach (DataSourceFactory *factory, m_dataSourceFactory)
@@ -385,12 +395,9 @@ void Metalink::processedSizeChanged()
     {
         m_percent = 0;
     }
-
-    Transfer::ChangesFlags flags = (Tc_DownloadedSize | Tc_Percent);
-    setTransferChange(flags, true);
 }
 
-void Metalink::speedChanged()
+void Metalink::recalculateSpeed()
 {
     m_downloadSpeed = 0;
     foreach (DataSourceFactory *factory, m_dataSourceFactory)
@@ -401,8 +408,6 @@ void Metalink::speedChanged()
         }
     }
 
-    setTransferChange(Tc_DownloadSpeed, true);
-
     //calculate the average of the last three speeds
     m_tempAverageSpeed += m_downloadSpeed;
     ++m_speedCount;
@@ -423,10 +428,10 @@ int Metalink::remainingTime() const
     return KIO::calculateRemainingSeconds(m_totalSize, m_downloadedSize, m_averageSpeed);
 }
 
-void Metalink::slotStatus(Job::Status status)
+void Metalink::updateStatus(DataSourceFactory *sender, bool *changeStatus)
 {
-    ChangesFlags flags = Tc_Status;
-    bool changeStatus = true;
+    Job::Status status = (sender ? sender->status() : Job::Stopped);
+    *changeStatus = true;
     switch (status)
     {
         case Job::Aborted:
@@ -435,12 +440,12 @@ void Metalink::slotStatus(Job::Status status)
             foreach (DataSourceFactory *factory, m_dataSourceFactory) {
                 //one factory is still running, do not change the status
                 if (factory->doDownload() && (factory->status() == Job::Running)) {
-                    changeStatus = false;
+                    *changeStatus = false;
                     ++m_currentFiles;
                 }
             }
 
-            if (changeStatus) {
+            if (*changeStatus) {
                 setStatus(status);
             }
             break;
@@ -457,13 +462,12 @@ void Metalink::slotStatus(Job::Status status)
                 //one factory is not finished, do not change the status
                 if (factory->doDownload() && (factory->status() != Job::Finished))
                 {
-                    changeStatus = false;
+                    *changeStatus = false;
                     break;
                 }
             }
 
-            if (changeStatus)
-            {
+            if (*changeStatus) {
                 setStatus(Job::Finished);
             }
             break;
@@ -473,20 +477,12 @@ void Metalink::slotStatus(Job::Status status)
             break;
     }
 
-    if (m_fileModel)
-    {
-        DataSourceFactory *factory = qobject_cast<DataSourceFactory*>(sender());
-        if (factory)
-        {
-            QModelIndex statusIndex = m_fileModel->index(factory->dest(), FileItem::Status);
+    if (m_fileModel) {
+        if (sender) {
+            QModelIndex statusIndex = m_fileModel->index(sender->dest(), FileItem::Status);
             m_fileModel->setData(statusIndex, status);
         }
     }
-
-    if (changeStatus)
-    {
-        setTransferChange(flags, true);
-    }
 }
 
 void Metalink::slotVerified(bool isVerified)
@@ -620,13 +616,12 @@ void Metalink::load(const QDomElement *element)
 
         DataSourceFactory *file = new DataSourceFactory(this);
         file->load(&factory);
-        connect(file, SIGNAL(totalSize(KIO::filesize_t)), this, SLOT(totalSizeChanged(KIO::filesize_t)));
-        connect(file, SIGNAL(processedSize(KIO::filesize_t)), this, SLOT(processedSizeChanged()));
-        connect(file, SIGNAL(speed(ulong)), this, SLOT(speedChanged()));
-        connect(file, SIGNAL(statusChanged(Job::Status)), this, SLOT(slotStatus(Job::Status)));
+        connect(file, SIGNAL(capabilitiesChanged()), this, SLOT(slotUpdateCapabilities()));
+        connect(file, SIGNAL(dataSourceFactoryChange(Transfer::ChangesFlags)), this, SLOT(slotDataSourceFactoryChange(Transfer::ChangesFlags)));
         m_dataSourceFactory[file->dest()] = file;
         connect(file->verifier(), SIGNAL(verified(bool)), this, SLOT(slotVerified(bool)));
         connect(file->signature(), SIGNAL(verified(int)), this, SLOT(slotSignatureVerified()));
+        connect(file, SIGNAL(log(QString,Transfer::LogLevel)), this, SLOT(setLog(QString,Transfer::LogLevel)));
 
         //start the DataSourceFactories that were Started when KGet was closed
         if (file->status() == Job::Running) {
@@ -640,6 +635,7 @@ void Metalink::load(const QDomElement *element)
         }
     }
     m_ready = !m_dataSourceFactory.isEmpty();
+    slotUpdateCapabilities();
 }
 
 void Metalink::save(const QDomElement &element)
@@ -709,6 +705,91 @@ FileModel *Metalink::fileModel()
     return m_fileModel;
 }
 
+void Metalink::filesSelected()
+{
+    bool overwriteAll = false;
+    bool autoSkip = false;
+    bool cancel = false;
+    QModelIndexList files = fileModel()->fileIndexes(FileItem::File);
+    m_numFilesSelected = 0;
+
+    //sets the CheckState of the fileModel to the according DataSourceFactories
+    //and asks the user if there are existing files already
+    foreach (const QModelIndex &index, files)
+    {
+        const KUrl dest = fileModel()->getUrl(index);
+        bool doDownload = index.data(Qt::CheckStateRole).toBool();
+        if (m_dataSourceFactory.contains(dest))
+        {
+            DataSourceFactory *factory = m_dataSourceFactory[dest];
+            //ignore finished transfers
+            if ((factory->status() == Job::Finished) || (factory->status() == Job::FinishedKeepAlive)) {
+                continue;
+            }
+
+            //check if the file at dest exists already and ask the user what to do in this case, ignore already running transfers
+            if (doDownload && (factory->status() != Job::Running) && QFile::exists(dest.toLocalFile())) {
+                //usere has chosen to skip all files that exist already before
+                if (autoSkip) {
+                    fileModel()->setData(index, Qt::Unchecked, Qt::CheckStateRole);
+                    doDownload = false;
+                //ask the user, unless he has choosen overwriteAll before
+                } else if (!overwriteAll) {
+                    KIO::RenameDialog dlg(0, i18n("File already exists"), index.data().toString(), dest, KIO::RenameDialog_Mode(KIO::M_MULTI | KIO::M_OVERWRITE | KIO::M_SKIP));
+                    const int result = dlg.exec();
+
+                    if (result == KIO::R_RENAME) {
+                        //no reason to use FileModel::rename() since the file does not exist yet, so simply skip it
+                        //avoids having to deal with signals
+                        const KUrl newDest = dlg.newDestUrl();
+                        factory->setDoDownload(doDownload);
+                        factory->setNewDestination(newDest);
+                        fileModel()->setData(index, newDest.fileName(), FileItem::File);
+                        ++m_numFilesSelected;
+
+                        m_dataSourceFactory.remove(dest);
+                        m_dataSourceFactory[newDest] = factory;
+                        continue;
+                    } else if (result == KIO::R_SKIP) {
+                        fileModel()->setData(index, Qt::Unchecked, Qt::CheckStateRole);
+                        doDownload = false;
+                    } else if (result == KIO::R_CANCEL) {
+                        cancel = true;
+                        break;
+                    } else if (result == KIO::R_AUTO_SKIP) {
+                        autoSkip = true;
+                        fileModel()->setData(index, Qt::Unchecked, Qt::CheckStateRole);
+                        doDownload = false;
+                    } else if (result == KIO::R_OVERWRITE_ALL) {
+                        overwriteAll = true;
+                    }
+                }
+            }
+
+            factory->setDoDownload(doDownload);
+            if (doDownload && (factory->status() != Finished) && (factory->status() != FinishedKeepAlive)) {
+                ++m_numFilesSelected;
+            }
+        }
+    }
+
+    //the user decided to cancel, so untick all files
+    if (cancel) {
+        m_numFilesSelected = 0;
+        untickAllFiles();
+        foreach (DataSourceFactory *factory, m_dataSourceFactory) {
+            factory->setDoDownload(false);
+        }
+    }
+
+    Transfer::ChangesFlags change = (Tc_TotalSize | Tc_DownloadSpeed);
+    //some files have been selected that are not finished yet, set them to stop if the transfer is not running (checked in slotStatus)
+    if (m_numFilesSelected) {
+        change |= Tc_Status;
+    }
+    slotDataSourceFactoryChange(change);
+}
+
 void Metalink::slotRename(const KUrl &oldUrl, const KUrl &newUrl)
 {
     if (!m_dataSourceFactory.contains(oldUrl))
@@ -720,10 +801,6 @@ void Metalink::slotRename(const KUrl &oldUrl, const KUrl &newUrl)
     m_dataSourceFactory.remove(oldUrl);
     m_dataSourceFactory[newUrl]->setNewDestination(newUrl);
 
-#ifdef HAVE_NEPOMUK
-    m_nepHandler->setDestinations(files());
-#endif //HAVE_NEPOMUK
-
     setTransferChange(Tc_FileName);
 }
 
@@ -755,12 +832,7 @@ bool Metalink::setDirectory(const KUrl &new_directory)
     }
     m_dataSourceFactory = newStorage;
 
-#ifdef HAVE_NEPOMUK
-    m_nepHandler->setDestinations(files());
-#endif //HAVE_NEPOMUK
-
     setTransferChange(Tc_FileName);
-
     return true;
 }
 
@@ -787,4 +859,23 @@ void Metalink::setAvailableMirrors(const KUrl &file, const QHash<KUrl, QPair<boo
     m_dataSourceFactory[file]->setMirrors(mirrors);
 }
 
+void Metalink::slotUpdateCapabilities()
+{
+    Capabilities oldCap = capabilities();
+    Capabilities newCap = 0;
+    foreach (DataSourceFactory *file, m_dataSourceFactory) {
+        if (file->doDownload()) {//FIXME when a download did not start yet it should be moveable!!//FIXME why not working, when only two connections?
+            if (newCap) {
+                newCap &= file->capabilities();
+            } else {
+                newCap = file->capabilities();
+            }
+        }
+    }
+
+    if (newCap != oldCap) {
+        setCapabilities(newCap);
+    }
+}
+
 #include "metalink.moc"
