@@ -11,29 +11,42 @@
 #define GROUPSETTINGSDIALOG_H
 
 #include "ui_groupsettingsdialog.h"
+#include "../core/basedialog.h"
 
 #include <KDialog>
 
+#ifdef HAVE_NEPOMUK
+namespace Nepomuk {
+    class TagWidget;
+}
+#endif
+
 class TransferGroupHandler;
 
-class GroupSettingsDialog : public KDialog
+class GroupSettingsDialog : public KGetSaveSizeDialog
 {
     Q_OBJECT
     public:
         GroupSettingsDialog(QWidget *parent, TransferGroupHandler *group);
         ~GroupSettingsDialog();
 
+        QSize sizeHint() const;
+
     private slots:
         void save();
 
     private:
-        TransferGroupHandler* m_group;
+#ifdef HAVE_NEPOMUK
+        void updateUsedTagsLineEdit();
+#endif //HAVE_NEPOMUK
 
-        QCheckBox *m_downloadCheck;
-        QCheckBox *m_uploadCheck;
-        QSpinBox *m_downloadBox;
-        QSpinBox *m_uploadBox;
-        KUrlRequester *m_defaultFolderRequester;
+    private:
+        TransferGroupHandler* m_group;
+        Ui::GroupSettingsDialog ui;
+
+#ifdef HAVE_NEPOMUK
+        Nepomuk::TagWidget *m_tagWidget;
+#endif //HAVE_NEPOMUK
 };
 
 #endif
