@@ -486,8 +486,8 @@ bool XmlContactStorage::parseMetaContact( Kopete::MetaContact *metaContact, cons
         metaContact->slotAllPluginsLoaded();
     else
         // When all plugins are loaded, set the source contact.
-        QObject::connect( Kopete::PluginManager::self(), SIGNAL( allPluginsLoaded() ),
-                          metaContact, SLOT( slotAllPluginsLoaded() ) );
+        QObject::connect( Kopete::PluginManager::self(), SIGNAL(allPluginsLoaded()),
+                          metaContact, SLOT(slotAllPluginsLoaded()) );
 
     metaContact->setLoading( false );
     return true;
@@ -562,8 +562,8 @@ bool XmlContactStorage::parseGroup(Kopete::Group *group, const QDomElement &elem
                 group->setDisplayName( QLatin1String( "Top-Level" ) );
                 break;
             default:
-                group->setDisplayName( i18n( "(Unnamed Group)" ) );
-                break;
+                group->setLoading( false );
+                return false;
         }
     }
 
