@@ -7,6 +7,7 @@
     Copyright (c) 2002-2004 by Olivier Goffart        <ogoffart@kde.org>
     Copyright (c) 2003      by Jason Keirstead        <jason@keirstead.org>
     Copyright (c) 2005      by Michaël Larouche      <larouche@kde.org>
+    Copyright (c) 2009      by Fabian Rami          <fabian.rami@wowcompany.com>
 
     Kopete    (c) 2002-2003 by the Kopete developers  <kopete-devel@kde.org>
 
@@ -310,6 +311,11 @@ signals:
 	 */
 	void eventNotification( const QString& notificationText);
 
+	/**
+	 * Signals that view for that chat session was activated
+	 */
+	void viewActivated( KopeteView* view );
+
 	/**
 	 * Signals that a message has changed its state.
 	 * The chat window connects to this signal to update the message in chat view.
@@ -322,6 +328,12 @@ signals:
 	 */
 	void photoChanged();
 
+    /**
+    * Toggle Graphic Override mode for plugin
+    * @param enable Toggle the overide
+    */
+    void toggleGraphicOverride(bool enable);
+
 public slots:
 	/**
 	 * @brief Got a typing notification from a user
