@@ -22,7 +22,11 @@
 
 #include "jabberclient.h"
 
+#include <kdebug.h>
+#include "kopetesockettimeoutwatcher.h"
+
 #include <QTimer>
+#include <QTcpSocket>
 #include <QRegExp>
 #include <QtCrypto>
 
@@ -31,7 +35,6 @@
 #include <jinglesessionmanager.h>
 #include <xmpp_tasks.h>
 
-#include "jabberconnector.h"
 #include "privacymanager.h"
 
 #define JABBER_PENALTY_TIME	2
@@ -65,7 +68,7 @@ public:
 	// XMPP backend
 	XMPP::Client *jabberClient;
 	XMPP::ClientStream *jabberClientStream;
-	JabberConnector *jabberClientConnector;
+	XMPP::AdvancedConnector *jabberClientConnector;
 	QCA::TLS *jabberTLS;
 	XMPP::QCATLSHandler *jabberTLSHandler;
 	QCA::Initializer qcaInit;
@@ -132,7 +135,7 @@ JabberClient::JabberClient ()
 	cleanUp ();
 
 	// initiate penalty timer
-	QTimer::singleShot ( JABBER_PENALTY_TIME * 1000, this, SLOT ( slotUpdatePenaltyTime () ) );
+	QTimer::singleShot ( JABBER_PENALTY_TIME * 1000, this, SLOT (slotUpdatePenaltyTime()) );
 
 }
 
@@ -197,7 +200,7 @@ void JabberClient::slotUpdatePenaltyTime ()
 	else
 		d->currentPenaltyTime = 0;
 
-	QTimer::singleShot ( JABBER_PENALTY_TIME * 1000, this, SLOT ( slotUpdatePenaltyTime () ) );
+	QTimer::singleShot ( JABBER_PENALTY_TIME * 1000, this, SLOT (slotUpdatePenaltyTime()) );
 
 }
 
@@ -242,7 +245,7 @@ XMPP::S5BServer *JabberClient::s5bServer ()
 	if ( !d->s5bServer )
 	{
 		d->s5bServer = new XMPP::S5BServer ();
-		QObject::connect ( d->s5bServer, SIGNAL ( destroyed () ), this, SLOT ( slotS5BServerGone () ) );
+		QObject::connect ( d->s5bServer, SIGNAL (destroyed()), this, SLOT (slotS5BServerGone()) );
 
 		/*
 		 * Try to start the server at the default port here.
@@ -552,7 +555,7 @@ XMPP::ClientStream *JabberClient::clientStream () const
 
 }
 
-JabberConnector *JabberClient::clientConnector () const
+XMPP::AdvancedConnector *JabberClient::clientConnector () const
 {
 
 	return d->jabberClientConnector;
@@ -625,7 +628,7 @@ JabberClient::ErrorCode JabberClient::connect ( const XMPP::Jid &jid, const QStr
 	 * This class uses KDE's socket code, which in turn makes use of
 	 * the global proxy settings.
 	 */
-	d->jabberClientConnector = new JabberConnector;
+	d->jabberClientConnector = new XMPP::AdvancedConnector;
 
 	d->jabberClientConnector->setOptSSL ( useSSL () );
 
@@ -650,7 +653,7 @@ JabberClient::ErrorCode JabberClient::connect ( const XMPP::Jid &jid, const QStr
 		d->jabberTLSHandler = new QCATLSHandler(d->jabberTLS);
 		d->jabberTLSHandler->setXMPPCertCheck(true);
 
-		QObject::connect ( d->jabberTLSHandler, SIGNAL ( tlsHandshaken() ), SLOT ( slotTLSHandshaken () ) );
+		QObject::connect ( d->jabberTLSHandler, SIGNAL (tlsHandshaken()), SLOT (slotTLSHandshaken()) );
 	}
 
 	/*
@@ -661,18 +664,20 @@ JabberClient::ErrorCode JabberClient::connect ( const XMPP::Jid &jid, const QStr
 
 	{
 		using namespace XMPP;
-		QObject::connect ( d->jabberClientStream, SIGNAL ( needAuthParams(bool, bool, bool) ),
-				   this, SLOT ( slotCSNeedAuthParams (bool, bool, bool) ) );
-		QObject::connect ( d->jabberClientStream, SIGNAL ( authenticated () ),
-				   this, SLOT ( slotCSAuthenticated () ) );
-		QObject::connect ( d->jabberClientStream, SIGNAL ( connectionClosed () ),
-				   this, SLOT ( slotCSDisconnected () ) );
-		QObject::connect ( d->jabberClientStream, SIGNAL ( delayedCloseFinished () ),
-				   this, SLOT ( slotCSDisconnected () ) );
-		QObject::connect ( d->jabberClientStream, SIGNAL ( warning (int) ),
-				   this, SLOT ( slotCSWarning (int) ) );
-		QObject::connect ( d->jabberClientStream, SIGNAL ( error (int) ),
-				   this, SLOT ( slotCSError (int) ) );
+		QObject::connect ( d->jabberClientStream, SIGNAL (needAuthParams(bool,bool,bool)),
+				   this, SLOT (slotCSNeedAuthParams(bool,bool,bool)) );
+		QObject::connect ( d->jabberClientStream, SIGNAL (authenticated()),
+				   this, SLOT (slotCSAuthenticated()) );
+		QObject::connect ( d->jabberClientStream, SIGNAL (connectionClosed()),
+				   this, SLOT (slotCSDisconnected()) );
+		QObject::connect ( d->jabberClientStream, SIGNAL (delayedCloseFinished()),
+				   this, SLOT (slotCSDisconnected()) );
+		QObject::connect ( d->jabberClientStream, SIGNAL (warning(int)),
+				   this, SLOT (slotCSWarning(int)) );
+		QObject::connect ( d->jabberClientStream, SIGNAL (error(int)),
+				   this, SLOT (slotCSError(int)) );
+		QObject::connect ( d->jabberClientStream, SIGNAL (connected()),
+		                   this, SLOT (slotCSConnected()) );
 	}
 
 	d->jabberClientStream->setOldOnly ( useXMPP09 () );
@@ -707,8 +712,8 @@ JabberClient::ErrorCode JabberClient::connect ( const XMPP::Jid &jid, const QStr
 
 		{
 			using namespace XMPP;
-			QObject::connect ( d->jabberClient->fileTransferManager(), SIGNAL ( incomingReady() ),
-					   this, SLOT ( slotIncomingFileTransfer () ) );
+			QObject::connect ( d->jabberClient->fileTransferManager(), SIGNAL (incomingReady()),
+					   this, SLOT (slotIncomingFileTransfer()) );
 		}
 	}
 
@@ -720,8 +725,8 @@ JabberClient::ErrorCode JabberClient::connect ( const XMPP::Jid &jid, const QStr
 		
 		{
 			using namespace XMPP;
-			QObject::connect ( d->jabberClient->jingleSessionManager(), SIGNAL ( incomingSession() ),
-					   this, SLOT ( slotIncomingJingleSession () ) );
+			QObject::connect ( d->jabberClient->jingleSessionManager(), SIGNAL (incomingSession()),
+					   this, SLOT (slotIncomingJingleSession()) );
 		}
 #else
 		d->jabberClient->setJingleEnabled(false);
@@ -733,36 +738,36 @@ JabberClient::ErrorCode JabberClient::connect ( const XMPP::Jid &jid, const QStr
 	 */
 	{
 		using namespace XMPP;
-		QObject::connect ( d->jabberClient, SIGNAL ( subscription (const Jid &, const QString &, const QString &) ),
-				   this, SLOT ( slotSubscription (const Jid &, const QString &) ) );
-		QObject::connect ( d->jabberClient, SIGNAL ( rosterRequestFinished ( bool, int, const QString & ) ),
-				   this, SLOT ( slotRosterRequestFinished ( bool, int, const QString & ) ) );
-		QObject::connect ( d->jabberClient, SIGNAL ( rosterItemAdded (const RosterItem &) ),
-				   this, SLOT ( slotNewContact (const RosterItem &) ) );
-		QObject::connect ( d->jabberClient, SIGNAL ( rosterItemUpdated (const RosterItem &) ),
-				   this, SLOT ( slotContactUpdated (const RosterItem &) ) );
-		QObject::connect ( d->jabberClient, SIGNAL ( rosterItemRemoved (const RosterItem &) ),
-				   this, SLOT ( slotContactDeleted (const RosterItem &) ) );
-		QObject::connect ( d->jabberClient, SIGNAL ( resourceAvailable (const Jid &, const Resource &) ),
-				   this, SLOT ( slotResourceAvailable (const Jid &, const Resource &) ) );
-		QObject::connect ( d->jabberClient, SIGNAL ( resourceUnavailable (const Jid &, const Resource &) ),
-				   this, SLOT ( slotResourceUnavailable (const Jid &, const Resource &) ) );
-		QObject::connect ( d->jabberClient, SIGNAL ( messageReceived (const Message &) ),
-				   this, SLOT ( slotReceivedMessage (const Message &) ) );
-		QObject::connect ( d->jabberClient, SIGNAL ( groupChatJoined (const Jid &) ),
-				   this, SLOT ( slotGroupChatJoined (const Jid &) ) );
-		QObject::connect ( d->jabberClient, SIGNAL ( groupChatLeft (const Jid &) ),
-				   this, SLOT ( slotGroupChatLeft (const Jid &) ) );
-		QObject::connect ( d->jabberClient, SIGNAL ( groupChatPresence (const Jid &, const Status &) ),
-				   this, SLOT ( slotGroupChatPresence (const Jid &, const Status &) ) );
-		QObject::connect ( d->jabberClient, SIGNAL ( groupChatError (const Jid &, int, const QString &) ),
-				   this, SLOT ( slotGroupChatError (const Jid &, int, const QString &) ) );
-		//QObject::connect ( d->jabberClient, SIGNAL (debugText (const QString &) ),
-		//		   this, SLOT ( slotPsiDebug (const QString &) ) );
-		QObject::connect ( d->jabberClient, SIGNAL ( xmlIncoming(const QString& ) ),
-				   this, SLOT ( slotIncomingXML (const QString &) ) );
-		QObject::connect ( d->jabberClient, SIGNAL ( xmlOutgoing(const QString& ) ),
-				   this, SLOT ( slotOutgoingXML (const QString &) ) );
+		QObject::connect ( d->jabberClient, SIGNAL (subscription(Jid,QString,QString)),
+				   this, SLOT (slotSubscription(Jid,QString)) );
+		QObject::connect ( d->jabberClient, SIGNAL (rosterRequestFinished(bool,int,QString)),
+				   this, SLOT (slotRosterRequestFinished(bool,int,QString)) );
+		QObject::connect ( d->jabberClient, SIGNAL (rosterItemAdded(RosterItem)),
+				   this, SLOT (slotNewContact(RosterItem)) );
+		QObject::connect ( d->jabberClient, SIGNAL (rosterItemUpdated(RosterItem)),
+				   this, SLOT (slotContactUpdated(RosterItem)) );
+		QObject::connect ( d->jabberClient, SIGNAL (rosterItemRemoved(RosterItem)),
+				   this, SLOT (slotContactDeleted(RosterItem)) );
+		QObject::connect ( d->jabberClient, SIGNAL (resourceAvailable(Jid,Resource)),
+				   this, SLOT (slotResourceAvailable(Jid,Resource)) );
+		QObject::connect ( d->jabberClient, SIGNAL (resourceUnavailable(Jid,Resource)),
+				   this, SLOT (slotResourceUnavailable(Jid,Resource)) );
+		QObject::connect ( d->jabberClient, SIGNAL (messageReceived(Message)),
+				   this, SLOT (slotReceivedMessage(Message)) );
+		QObject::connect ( d->jabberClient, SIGNAL (groupChatJoined(Jid)),
+				   this, SLOT (slotGroupChatJoined(Jid)) );
+		QObject::connect ( d->jabberClient, SIGNAL (groupChatLeft(Jid)),
+				   this, SLOT (slotGroupChatLeft(Jid)) );
+		QObject::connect ( d->jabberClient, SIGNAL (groupChatPresence(Jid,Status)),
+				   this, SLOT (slotGroupChatPresence(Jid,Status)) );
+		QObject::connect ( d->jabberClient, SIGNAL (groupChatError(Jid,int,QString)),
+				   this, SLOT (slotGroupChatError(Jid,int,QString)) );
+		//QObject::connect ( d->jabberClient, SIGNAL (debugText(QString)),
+		//		   this, SLOT (slotPsiDebug(QString)) );
+		QObject::connect ( d->jabberClient, SIGNAL (xmlIncoming(QString)),
+				   this, SLOT (slotIncomingXML(QString)) );
+		QObject::connect ( d->jabberClient, SIGNAL (xmlOutgoing(QString)),
+				   this, SLOT (slotOutgoingXML(QString)) );
 	}
 
 	d->jabberClient->setClientName ( clientName () );
@@ -778,6 +783,9 @@ JabberClient::ErrorCode JabberClient::connect ( const XMPP::Jid &jid, const QStr
 
 	d->jabberClient->setTimeZone ( timeZoneName (), timeZoneOffset () );
 
+	// Additional features
+	d->jabberClient->setFeatures(Features("http://jabber.org/protocol/xhtml-im"));
+
 	d->jabberClient->connectToServer ( d->jabberClientStream, jid, auth );
 
 	return Ok;
@@ -998,13 +1006,6 @@ void JabberClient::slotCSAuthenticated ()
 		{
 			d->localAddress = ( (BSocket *)irisByteStream )->address().toString ();
 		}
-
-		// code for the KDE-type bytestream
-		JabberByteStream *kdeByteStream = dynamic_cast<JabberByteStream*>(d->jabberClientConnector->stream());
-		if ( kdeByteStream )
-		{
-			d->localAddress = kdeByteStream->socket()->peerName();
-		}
 	}
 
 	if ( fileTransfersEnabled () )
@@ -1014,12 +1015,35 @@ void JabberClient::slotCSAuthenticated ()
 		d->jabberClient->s5bManager()->setServer ( s5bServer () );
 	}
 
+	
+	//update the ressource:
+	d->jid = d->jabberClientStream->jid();
+
 	// start the client operation
 	d->jabberClient->start ( jid().domain (), jid().node (), d->password, jid().resource () );
 
-	emit connected ();
+	if (!d->jabberClientStream->old())
+	{
+		XMPP::JT_Session *j = new XMPP::JT_Session(rootTask());
+		QObject::connect(j, SIGNAL(finished()), this, SLOT(slotSessionStarted()));
+		j->go(true);
+	}
+	else
+	{
+		emit connected();
+	}
 }
 
+void JabberClient::slotSessionStarted()
+{
+	XMPP::JT_Session *j = static_cast<XMPP::JT_Session*>(sender());
+	if ( j->success() )
+		emit connected();
+	else
+		emit csError ( -1 );
+}
+
+
 void JabberClient::slotCSDisconnected ()
 {
 
@@ -1072,6 +1096,18 @@ void JabberClient::slotCSError ( int error )
 
 }
 
+void JabberClient::slotCSConnected ()
+{
+	Kopete::SocketTimeoutWatcher* timeoutWatcher = 0;
+
+	ByteStream *irisByteStream = d->jabberClientConnector->stream ();
+	if ( irisByteStream && irisByteStream->abstractSocket () )
+		timeoutWatcher = Kopete::SocketTimeoutWatcher::watch ( irisByteStream->abstractSocket () );
+
+	if ( timeoutWatcher )
+		QObject::connect( timeoutWatcher, SIGNAL(errorInt(int)), this, SLOT(slotCSError(int)) );
+}
+
 void JabberClient::slotRosterRequestFinished ( bool success, int /*statusCode*/, const QString &/*statusString*/ )
 {
 
