@@ -97,19 +97,19 @@ JabberContact::JabberContact (const XMPP::RosterItem &rosterItem, Kopete::Accoun
 	{
 		// this contact is a regular contact
 		connect ( this,
-				  SIGNAL ( onlineStatusChanged ( Kopete::Contact *, const Kopete::OnlineStatus &, const Kopete::OnlineStatus & ) ),
-				  this, SLOT ( slotCheckVCard () ) );
+				  SIGNAL (onlineStatusChanged(Kopete::Contact*,Kopete::OnlineStatus,Kopete::OnlineStatus)),
+				  this, SLOT (slotCheckVCard()) );
 	}
 	else
 	{
 		// this contact is the myself instance
 		connect ( account()->myself (),
-				  SIGNAL ( onlineStatusChanged ( Kopete::Contact *, const Kopete::OnlineStatus &, const Kopete::OnlineStatus & ) ),
-				  this, SLOT ( slotCheckVCard () ) );
+				  SIGNAL (onlineStatusChanged(Kopete::Contact*,Kopete::OnlineStatus,Kopete::OnlineStatus)),
+				  this, SLOT (slotCheckVCard()) );
 
 		connect ( account()->myself (),
-				  SIGNAL ( onlineStatusChanged ( Kopete::Contact *, const Kopete::OnlineStatus &, const Kopete::OnlineStatus & ) ),
-				  this, SLOT ( slotCheckLastActivity ( Kopete::Contact *, const Kopete::OnlineStatus &, const Kopete::OnlineStatus & ) ) );
+				  SIGNAL (onlineStatusChanged(Kopete::Contact*,Kopete::OnlineStatus,Kopete::OnlineStatus)),
+				  this, SLOT (slotCheckLastActivity(Kopete::Contact*,Kopete::OnlineStatus,Kopete::OnlineStatus)) );
 
 		/*
 		 * Trigger update once if we're already connected for contacts
@@ -117,7 +117,8 @@ JabberContact::JabberContact (const XMPP::RosterItem &rosterItem, Kopete::Accoun
 		 */
 		if ( account()->myself()->onlineStatus().isDefinitelyOnline() )
 		{
-			slotGetTimedVCard ();
+			mVCardUpdateInProgress = true;
+			QTimer::singleShot ( 1000, this, SLOT (slotGetTimedVCard()) );
 		}
 	}
 
@@ -162,6 +163,22 @@ QList<KAction*> *JabberContact::customContextMenuActions ()
 	connect(removeAuthAction, SIGNAL(triggered(bool)), SLOT(slotRemoveAuth()));
 	actionAuthorization->addAction(removeAuthAction);
 
+#ifdef GOOGLETALK_SUPPORT
+
+	if ( account()->enabledGoogleTalk() ) {
+
+		KAction *googleTalkCallAction;
+		googleTalkCallAction = new KAction( this );
+		googleTalkCallAction->setIcon( (KIcon("voicecall") ) );
+		googleTalkCallAction->setText( i18n ("Call Google Talk contact") );
+		googleTalkCallAction->setEnabled( account()->supportGoogleTalk(contactId()) );
+		connect(googleTalkCallAction, SIGNAL(triggered(bool)), SLOT(makeGoogleTalkCallAction()));
+		actions->append(googleTalkCallAction);
+
+	}
+
+#endif
+
 	KActionMenu *actionSetAvailability = new KActionMenu ( KIcon("user-identity", 0, QStringList() << QString() << "user-online"), i18n ("Set Availability"), this );
 
 #define KACTION(status, text, name, slot) \
@@ -299,8 +316,8 @@ void JabberContact::handleIncomingMessage (const XMPP::Message & message)
 			QString originalBody=message.body().isEmpty() ? QString() :
 					i18n( "The original message is : <i>\" %1 \"</i><br />" , Qt::escape(message.body()));
 			QString mes=i18n("<qt><i>%1</i> has invited you to join the conference <b>%2</b><br />%3<br />"
-					"If you want to accept and join, just <b>enter your nickname</b> and press ok<br />"
-							 "If you want to decline, press cancel</qt>",
+					"If you want to accept and join, just <b>enter your nickname</b> and press OK.<br />"
+							 "If you want to decline, press Cancel.</qt>",
 					message.from().full(), room , originalBody);
 			
 			bool ok=false;
@@ -310,7 +327,7 @@ void JabberContact::handleIncomingMessage (const XMPP::Message & message)
 				return;
 			
 			XMPP::Jid roomjid(room);
-			account()->client()->joinGroupChat( roomjid.host() , roomjid.user() , futureNewNickName );
+			account()->client()->joinGroupChat( roomjid.domain() , roomjid.node() , futureNewNickName );
 			return;
 		}
 		else if (message.body().isEmpty())
@@ -340,8 +357,12 @@ void JabberContact::handleIncomingMessage (const XMPP::Message & message)
 					Kopete::Message m=Kopete::Message ( this, mManager->members() );
 					m.setPlainBody( i18n("%1 has ended his/her participation in the chat session.", metaContact()->displayName()) );
 					m.setDirection( Kopete::Message::Internal );
+					m.setImportance(Kopete::Message::Low);
 
-					mManager->appendMessage ( m, message.from().resource () );
+					if ( account()->mergeMessages() )
+						mManager->appendMessage ( m, QString() );
+					else
+						mManager->appendMessage ( m, message.from().resource () );
 				}
 			}
 		}
@@ -426,7 +447,10 @@ void JabberContact::handleIncomingMessage (const XMPP::Message & message)
 	// append message to (eventually new) manager and preselect the originating resource
 	if ( newMessage )
 	{
-		mManager->appendMessage ( *newMessage, message.from().resource () );
+		if ( account()->mergeMessages() )
+			mManager->appendMessage ( *newMessage, QString() );
+		else
+			mManager->appendMessage ( *newMessage, message.from().resource () );
 
 		delete newMessage;
 	}
@@ -450,7 +474,10 @@ void JabberContact::handleIncomingMessage (const XMPP::Message & message)
 		msg.setDirection( Kopete::Message::Inbound );
 		msg.setRequestedPlugin( viewPlugin );
 
-		mManager->appendMessage ( msg, message.from().resource () );
+		if ( account()->mergeMessages() )
+			mManager->appendMessage ( msg, QString() );
+		else
+			mManager->appendMessage ( msg, message.from().resource () );
 	}
 }
 
@@ -498,7 +525,7 @@ void JabberContact::slotCheckVCard ()
 		mVCardUpdateInProgress = true;
 
 		// current data is older than 24 hours, request a new one
-		QTimer::singleShot ( account()->client()->getPenaltyTime () * 1000, this, SLOT ( slotGetTimedVCard () ) );
+		QTimer::singleShot ( account()->client()->getPenaltyTime () * 1000, this, SLOT (slotGetTimedVCard()) );
 	}
 
 }
@@ -537,7 +564,7 @@ void JabberContact::slotGetTimedVCard ()
 	// request vCard
 	XMPP::JT_VCard *task = new XMPP::JT_VCard ( account()->client()->rootTask () );
 	// signal to ourselves when the vCard data arrived
-	QObject::connect ( task, SIGNAL ( finished () ), this, SLOT ( slotGotVCard () ) );
+	QObject::connect ( task, SIGNAL (finished()), this, SLOT (slotGotVCard()) );
 	task->get ( mRosterItem.jid () );
 	task->go ( true );
 
@@ -596,7 +623,7 @@ void JabberContact::slotCheckLastActivity ( Kopete::Contact *, const Kopete::Onl
 	{
 		kDebug ( JABBER_DEBUG_GLOBAL ) << "Scheduling request for last activity for " << mRosterItem.jid().bare ();
 
-		QTimer::singleShot ( account()->client()->getPenaltyTime () * 1000, this, SLOT ( slotGetTimedLastActivity () ) );
+		QTimer::singleShot ( account()->client()->getPenaltyTime () * 1000, this, SLOT (slotGetTimedLastActivity()) );
 	}
 
 }
@@ -623,7 +650,7 @@ void JabberContact::slotGetTimedLastActivity ()
 		kDebug ( JABBER_DEBUG_GLOBAL ) << "Requesting last activity from timer for " << mRosterItem.jid().bare ();
 
 		JT_GetLastActivity *task = new JT_GetLastActivity ( account()->client()->rootTask () );
-		QObject::connect ( task, SIGNAL ( finished () ), this, SLOT ( slotGotLastActivity () ) );
+		QObject::connect ( task, SIGNAL (finished()), this, SLOT (slotGotLastActivity()) );
 		task->get ( mRosterItem.jid () );
 		task->go ( true );
 	}
@@ -762,7 +789,7 @@ void JabberContact::slotSendVCard()
 
 	XMPP::JT_VCard *task = new XMPP::JT_VCard (account()->client()->rootTask ());
 	// signal to ourselves when the vCard data arrived
-	QObject::connect (task, SIGNAL (finished ()), this, SLOT (slotSentVCard ()));
+	QObject::connect (task, SIGNAL (finished()), this, SLOT (slotSentVCard()));
 	task->set (rosterItem().jid(), vCard);
 	task->go (true);
 }
@@ -866,12 +893,12 @@ JabberChatSession *JabberContact::manager ( Kopete::ContactPtrList chatMembers,
 		 * will stay empty.
 		 */
 		if ( jid.resource().isEmpty () )
-			jid.setResource ( account()->resourcePool()->lockedResource ( jid ).name () );
+			jid = jid.withResource ( account()->resourcePool()->lockedResource ( jid ).name () );
 
 		kDebug(JABBER_DEBUG_GLOBAL) << "No manager found, creating a new one with resource '" << jid.resource () << "'";
 
 		manager = new JabberChatSession ( protocol(), static_cast<JabberBaseContact *>(account()->myself()), chatMembers, jid.resource () );
-		connect ( manager, SIGNAL ( destroyed ( QObject * ) ), this, SLOT ( slotChatSessionDeleted ( QObject * ) ) );
+		connect ( manager, SIGNAL (destroyed(QObject*)), this, SLOT (slotChatSessionDeleted(QObject*)) );
 		mManagers.append ( manager );
 	}
 
@@ -928,7 +955,7 @@ JabberChatSession *JabberContact::manager ( const QString &resource, Kopete::Con
 		JabberChatSession *manager = new JabberChatSession ( protocol(),
 																   static_cast<JabberBaseContact *>(account()->myself()),
 																   chatmembers, resource );
-		connect ( manager, SIGNAL ( destroyed ( QObject * ) ), this, SLOT ( slotChatSessionDeleted ( QObject * ) ) );
+		connect ( manager, SIGNAL (destroyed(QObject*)), this, SLOT (slotChatSessionDeleted(QObject*)) );
 		mManagers.append ( manager );
 
 		return manager;
@@ -1386,4 +1413,13 @@ void JabberContact::startJingleVideoCall()
 }
 #endif
 
+#ifdef GOOGLETALK_SUPPORT
+
+void JabberContact::makeGoogleTalkCallAction()
+{
+	account()->makeGoogleTalkCall(contactId());
+}
+
+#endif
+
 #include "jabbercontact.moc"
